package checkers.nonnull;

import java.util.Properties;

import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.AnnotationMirror;

import com.sun.source.tree.CompilationUnitTree;

import checkers.basetype.BaseTypeChecker;
import checkers.basetype.BaseTypeVisitor;
import checkers.quals.NonNull;
import checkers.source.*;
import checkers.types.AnnotatedTypeFactory;
import checkers.types.AnnotatedTypeMirror;
import checkers.types.AnnotationFactory;
import checkers.util.*;

/**
 * A typechecker plug-in for the {@link NonNull} qualifier that finds (and
 * verifies the absence of) null-pointer errors.
 */
@SupportedAnnotationTypes({"checkers.quals.NonNull", "checkers.quals.Nullable"})
@SupportedLintOptions({"flow", "cast", "cast:redundant"})
@SupportedSourceVersion(SourceVersion.RELEASE_7)
@SuppressWarningsKey("nonnull")
public class NonNullChecker extends BaseTypeChecker {

    private AnnotationFactory annoFactory;
    private AnnotationMirror NULLABLE;
    private AnnotationMirror NONNULL;
    private SimpleSubtypeRelation relation;
    
    @Override
    public synchronized void init(ProcessingEnvironment processingEnv) {
        super.init(processingEnv);
        annoFactory = new AnnotationFactory(processingEnv);
        NULLABLE = this.annoFactory.fromName("checkers.quals.Nullable");
        NONNULL = this.annoFactory.fromName("checkers.quals.NonNull");
    }
    
    @Override
    public boolean isSubtype(AnnotatedTypeMirror lhs, AnnotatedTypeMirror rhs) {
        return this.relation.isSubtype(lhs, rhs);
    }
    
    @Override
    protected Properties getMessages() {
        Properties msgs = new Properties(super.getMessages());
        msgs.setProperty("dereference.of.nullable",
                "cannot dereference the possibly-null value %s");
        return msgs;
    }

    @Override
    protected BaseTypeVisitor<?, ?> getSourceVisitor(CompilationUnitTree root) {
        return new NonNullVisitor(this, root);
    }

    @Override
    public AnnotatedTypeFactory getFactory(ProcessingEnvironment env,
            CompilationUnitTree root) {
        AnnotatedTypeFactory factory = new NonNullAnnotatedTypeFactory(env, root); 
        relation = new SimpleSubtypeRelation(NONNULL, NULLABLE, new AnnotatedTypes(env, factory));
        return factory;
    }
    
    
}
