package checkers.util.test;

import javax.annotation.processing.*;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.AnnotationMirror;

import com.sun.source.tree.CompilationUnitTree;

import checkers.basetype.*;
import checkers.types.*;
import checkers.util.*;

/**
 * A simple checker used for testing the checkers framework.  It treates the
 * {@code \@Odd} annotation as a subtype-style qualifier with no special
 * semantics.
 *
 * <p>
 *
 * This checker should only be used for testing the framework.
 */
@SupportedSourceVersion(SourceVersion.RELEASE_7)
public class TestChecker extends BaseTypeChecker {

    private SimpleSubtypeRelation relation;

    protected AnnotationFactory annoFactory;

    /** Represents the {@code \@Odd} annotation. */
    protected AnnotationMirror ODD;

    @Override
    public synchronized void init(ProcessingEnvironment processingEnv) {
        super.init(processingEnv);
        annoFactory = new AnnotationFactory(processingEnv);
        ODD = this.annoFactory.fromName(Odd.class.getCanonicalName());
    }

    @Override
    public AnnotatedTypeFactory getFactory(ProcessingEnvironment env,
            CompilationUnitTree root) {
        AnnotatedTypeFactory factory = super.getFactory(env, root);
        this.relation = new SimpleSubtypeRelation(ODD, null, new AnnotatedTypes(env, factory));
        return factory;
    }

    @Override
    public boolean isSubtype(AnnotatedTypeMirror lhs, AnnotatedTypeMirror rhs) {
        return relation.isSubtype(lhs, rhs);
    }
}
